package node;

import clientserver.*;

import java.io.*;
import java.util.*;

public class AgentClassLoaderDynamic extends ClassLoader {
    // The classloader has no special functionality, it solely identifies
    // the source of the agents.
    // The agents and its helper classes  are 'pushed' inside the loader
    // upon an EXEC_REQ.
    Hashtable cache;
    String packageName;
    String senderID;
    Hashtable loaded=new Hashtable();

    public AgentClassLoaderDynamic(String id) {
	senderID=id;
	cache=new Hashtable();
    }

    public String getContext() {
	return senderID;
    }

    public synchronized Class loadClass(String name, boolean resolve)
    throws ClassNotFoundException {

	//System.out.println("Entering lookup for class "+name);

	Class c= (Class) cache.get(name);
	// Look in the cache (never load a class twice by the same loader).
	
	if (c==null) {
	    // Don't let the loader override system classes.
	    try {
		
		//System.out.print(" ..looking in sys.. ");
		c=findSystemClass(name); 
		//if(c!=null) System.out.println("Found "+c+" in the system.");
	    } catch (ClassNotFoundException e){}
	}
	if (c==null) { // Look in the loaded classes.
	    //System.out.print(" ..looking in loaded.. ");

	    ByteArray ba=(ByteArray) loaded.get(name);
	    if (ba==null) {
		throw(new ClassNotFoundException(name));
	    }
	    byte[] bytes=ba.bytes;
	    c=defineClass(name, bytes,0,bytes.length);
	    //System.out.println("Found in loaded");

	    //c=defineClass(b,0,b.length);

	    cache.put(name,c);

	}
	if(resolve) {
	    //System.out.println("Resolving: "+c);
	    resolveClass(c);
	}
	//System.out.println("Returning "+c);
	return c;
    }


    public ErrorMessage pushClasses(Vector names, Vector byteArrays) {

	Object o;
	ByteArray ba=null;
	String name;
	ErrorMessage error=null;
	
	if (names.size() != byteArrays.size()) {
	    error=new ErrorMessage("Different number of class names and byte arrays.");
	    return error;
	}
	for(Enumeration be=byteArrays.elements(), ne=names.elements(); be.hasMoreElements();) { //  Push each specified classname with the corresponding byte code.

	    o=ne.nextElement();	    
	    if (! (o instanceof String)) {
		error=new ErrorMessage("Classname "+o+" not stored as String.");
		return error;
	    }
	    //name=fullClassNameFor(o);
	    name=(String) o; // We already receive the full name 

	    o=be.nextElement();	    
	    if (! (o instanceof ByteArray)) {
		error=new ErrorMessage("Class "+o+" not stored as ByteArray.");
		return error;
	    }
	    ba=(ByteArray)  o;

	    //System.out.println("Pushing  class "+name);
	    loaded.put(name, ba);
	}
	return error;
    }  

//     public String fullClassNameFor(Object o) {
// 	// Appends the package name, does the casting for pushClasses
// 	String r=(String) o;
// 	return packageName+'.'+r;
//     }


}
