package node;

import capsule.*;
import filter.*;
import clientserver.*;

public class ExecutionEnvironment {
    // A container for all instances relevant to isolate the agent plus the agent.

    protected String owner;
    protected int agentID;
    protected Agent agent;
    protected int hopCount;

    protected Policy policy;
 
    protected Filter filter; //
    protected ServiceChannel serviceChannel; 
    protected AgentWrapper agentwrapper; // Holds the thread of the agent.
    protected NodeServiceDelivery nsDelivery;
    protected Receiver receiver;

    protected ServerPOSenderReceiver posr; // So that the HomeChannel service can bi initialized.

    public ExecutionEnvironment(String o, int id, Agent ag, int hopC , Filter f,ServiceChannel rc, AgentWrapper aw, NodeServiceDelivery nsd, Receiver r, ServerPOSenderReceiver pr, Policy plcy) {
	owner=o; agentID=id; agent=ag; hopCount=hopC; filter=f; serviceChannel=rc; agentwrapper=aw; nsDelivery=nsd; receiver=r; posr=pr; policy=plcy;
    }

    public void start(T_Configurator config) {
	// Watch the order (servers should be there, when clients start).
	Thread thr=new Thread(serviceChannel);
	thr.start();
	serviceChannel.setThread(thr);
	thr=new Thread(nsDelivery);
	nsDelivery.setThread(thr);
	thr.start();
	thr=new Thread(agentwrapper);
	agentwrapper.setThread(thr);
	thr.start();
	thr=new Thread(receiver);
	receiver.setThread(thr);
	config.vrConnect(); // If the node listens on a vr, the receiver gets an open socket here.
	thr.start();

    }

    public void dispose(Result finalMessage) {
	// Stop the execution environment.
	((ServiceChannelV1) serviceChannel).finalMessage(finalMessage);
	receiver.stop(false);
	agentwrapper.stop(false);
	nsDelivery.stop(false);
	serviceChannel.stop(false);
	//sendFinalMessage(finalMessage);
    }

    public String owner() {return owner;}
    public Agent agent() {return agent;}
    public int agentID() {return agentID;}
    public int hopCount() {return hopCount;}
    public Filter filter() {return filter;}
    public ServiceChannel serviceChannel() {return serviceChannel;}
    public NodeServiceDelivery serviceDelivery() {return nsDelivery;}
    public AgentWrapper agentWrapper() {return agentwrapper;}
    public Receiver receiver() {return receiver;}
    public ServerPOSenderReceiver posr() {return posr;}
    public Policy policy() {return policy;}



//     protected void sendFinalMessage(Result finalMessage) {
// // 	if(finalMessage!=null) {
// // 	    (new Thread(new FinalMessageSender(posr, finalMessage))).start();
// // 	    // Must be done in own thread in order not to produce a deadlock.
// // 	}
// 	if(finalMessage!=null) {
// 	    if(!posr.finished()) {
// 		System.err.println("SSS");
// 		posr.send(finalMessage);
// 	    }
// 	}
//     }


    public boolean sameAgent(ExecutionEnvironment ev) {
	// To avoid being flooded by the same agent.
	
	return (owner.equals(ev.owner)) && (agentID==ev.agentID);
    }

    public String toString() {
	// Important for the queries, because that is an answer to one.
	StringBuffer buf=new StringBuffer();
	buf.append("Execution environment for agent:\n");
	buf.append(agent.getClass().toString()); // A direct call is
	// a security problem. 'getClass()' however is final.
	buf.append(" of: ");
	buf.append(owner);
	buf.append("\nwith filter: ");
	buf.append(filter.toString());
	return new String(buf);
    }

}
