package node;

import java.io.*;
import utils.AddressCalculator;

public class IPPacket implements Serializable {
    // Optimized. Keeps the bytearray which got read in one single function
    // call. offers direct access to the array. Assumed that the user
    // (agents) uses each field only once, there is no overhead.
    // Else a special subclass could cache the field values instead of
    // calculating them from the byte array.
    // Added array boundary security check for all getter methods.. maybe a bit slower.

    public byte version(){ // 4bits (0..15)
	return (byte) ((bytes(0)&243) >> 4);
    }
    public byte headerLen(){ // 4bits (0..15);
	// Header length in words (4 bytes), default is 5.
	return (byte) (bytes(0)&15);
    }
    public byte DSCP() {
	return bytes(1);
    }
    public int len() { // only 2 bytes!
	return toNumber(2,2);
    }

    public int fragment() {
	return toNumber(4,4);
    } //a word of data for fragmentation ...stored but ignored.
    public byte ttl() {
	return bytes(8);
    }
    public byte proto() {
	return bytes(9);
    }

    public short headerCheckSum() {
	return (short)toNumber(10,2);
    } 

    public int source() {
	return toNumber(12,4);
    }
    public int destination(){
	return toNumber(16,4);
    }

    public byte[] options() {
	byte[] opts=new byte[optLen()];
	System.arraycopy(bytes,20,opts,0,optLen());
	return opts;
    }

    // convenient ways to get full access
    public byte dataByteAt(int pos) {
	return bytes(pos+headerLen()*4);
    } 

    public byte[] bytes; // the backdoor (for high performance).

    public byte bytes(int pos) {
	//secure access
	if ((pos<bytes.length) &&(pos >-1))
	    return bytes[pos];
	else {
	    //System.err.print(" !"+bytes.length);
	}
	return 0;
    }

    public long timeStamp; // For measuring; is the time provided by tcpdump

    public IPPacket(byte[] b, long ts) {
	bytes=b;
	timeStamp=ts;
	//nsistencyCheck(); // this check is outsourced to IPConsistencyMeter to keep IPPacket lean&fast.
    }


    public String toString() {
	StringBuffer buf=new StringBuffer();
	buf.append("IP Packet from: ");
	buf.append(source());
	buf.append(" to: ");
	buf.append(destination());
	buf.append(" data: \n");
	for (int i=20; i< bytes.length; i++) {// not so good; should respect header opts.
	    buf.append((char) bytes(i));
	}
	return new String(buf);
    }


    public int optLen() {
	// Length of ip options in bytes
	return 4*(headerLen()-5); 
    }

    public int toBigEndianNumber(int from, int len) {
	if ((from<0)|| (from+len>=bytes.length)) return 0;
	return AddressCalculator.toBigEndInt(bytes, from, len);
    }

    public int toNumber(int from, int len) {
	// Helper method to extract 1-4 bytes a time.
	// Preserves 2-complement
	if ((from<0)|| (from+len>=bytes.length)) return 0;
	return AddressCalculator.toInt(bytes, from, len); // len 4 is hardcoded there.

	//Buggy:
// 	int ret=0;
// 	int to=from+len;
// 	int i=from;
// 	while(i<65536  ) {
// 	    ret+= bytes[i]; // this smells like trouble!
// 	    i++;
// 	    if (i<to) ret<<=8;
// 	    else break;
// 	}
// 	return ret;
    }

    public int toPositiveNumber(int from, int len) {
	// warning: should not be used for more than 3 bytes.
	// is used for the checksum calculation.
	long ret=0;
	int b;
	int to=from+len;
	while(true) {
	    b=bytes(from);
	    if (b<0) b+=256; // assuming 2-complement.
	    ret+=b;
	    from++;
	    if (from<to) ret<<=8;
	    else break;
	}
	return (int) ret;	
    }

}
