package node;

import filter.*;
import java.util.*;


public class Policy {
    // Describes an upper limit of ressources for a specific user's agent.
    // Implements the policy conformity checks.

    protected Vector filters; // the filter used must be 'within' one of these filters
    protected boolean mustAuth;
    protected boolean superuser;
    protected int time2live;
    protected long maxMem;
    protected long maxExecTime;
    protected short priority;

    protected Vector services; // Numbers of the non-default node services available.

    public Policy(Vector v,int ttl, long m , long e, short p,  boolean ma, boolean su, Vector ser) {
	filters=v; time2live=ttl; maxMem=m; maxExecTime=e;
	mustAuth=ma; superuser=su;
	priority=p; services=ser;
	if (filters.size()<1) {
	    System.err.println("Warning! Policy does not define a filter!");
	}
    }

    public Filter getFirstMatchingFilter(Filter requested) {
	// Returns the first filter in the vector which is not empty when cut
	// with the requested filter. Otherwise returns null.
	Filter ret,pf;
	
	if(requested==null) {
	    System.err.println("Warning: got 'null' instead of a filter request.");
	    return null;// (Filter) filters.firstElement();
	}

	for (Enumeration e=filters.elements(); e.hasMoreElements();) {
	    pf=(Filter) e.nextElement();
	    ret=pf.and(requested); // Do the cut.
	    if(!ret.isEmpty()) return ret;
	}
	return null;
    }
    
//     public boolean checkConformity(RessourceUsage r) {
// 	// Checks the variable part of the used ressources.
// 	// Must be repeated from time to time.
// 	// <Unused>
	
// 	if (r.executionTime()>maxCPU) return false;
// 	if (System.currentTimeMillis()-r.startingTime()>maxExecTime) return false;
// 	if (r.memory()>maxMem) return false;
// 	return true;
//     }

    public boolean mustAuthenticate() {
	return mustAuth;
    }

    public boolean isSuperUser() {
	return superuser;
    }

    public long maxExecTime() {
	return maxExecTime;
    }

    public long maxMem() {
	return maxMem;
    }

    public int time2live() { // In secs
	return time2live;
    }

    public short priority() { 
	return priority;
    }

    public Vector services() { 
	return services;
    }

    public String toString() {
	StringBuffer buf=new StringBuffer(128);
	buf.append("POLICY: \nFilters:");
	buf.append(filters.toString());
	buf.append('\n');
	buf.append("MaxExecTime: "+maxExecTime);
	buf.append('\n');
	buf.append("Special services: "+services);
	buf.append( "\nauthentication is ");
	if(!mustAuth) buf.append( "not ");
	buf.append( "mandatory.");
	return new String(buf);
    }

}
