package node;

import java.io.Serializable;

import filter.*;
import java.util.*;

public abstract class PolicyGenerator implements Serializable{
    // Encodes a certain policy type. Subclass this class to implement
    // new policy types.

    // Programming policy classes instead of writing them in a special format
    // is a more general solution.

    // Some values are determined by the subclasses (via setting the
    // instance variables or overriding the creation methods; others
    // are directly set from the user profile (class of users).

    protected boolean isSuper=true;  // Is this a policy for the super user?
    protected boolean mustAuth; // Must the user authenticate to get this policy?

    protected  int time2live; //Maximum time an agent can stay in the node.
    protected  long maxMem; // The max size of an agent in bytes.
    protected  long execTime; // The max time  an agent can stay in the node.
    
    protected Node node; // The policy generator can make use of the registry and
    // gets the max values from the node.

    protected short priority; // Currently we use walues between 0..100, the
    // higher the better. 

    public Policy getPolicyFor(Profile p) {
	Vector v=createFilterVector(p);
	return new Policy(v, time2live , maxMem, execTime, priority ,mustAuth, isSuper, p.services()) ;
    }

    protected abstract Vector createFilterVector(Profile p);

    protected void init(Node node) {
	this.node=node;
	time2live=node.TIME_TO_LIVE(); // In secs.
	maxMem=node.MAX_MEM(); // The max size of an agent in bytes.
	execTime=node.MAX_EXEC_TIME(); // In millisecs
    }

}
