package node;

import java.io.*;
import java.util.*;

//import topology.NeighborRecord;
import clientserver.*;

public class QueryHandler {
    // The server side

    protected ServerPOSenderReceiver posr;
    protected ErrorMessage error=null;
    protected Node node; 

    public  QueryHandler(Node n, String senderID, DecodingResult dec, Query req, ServerPOSenderReceiver p) {
	node=n;
	posr=p;

	handleRequest( senderID, dec,  req);
    }

    protected synchronized void handleRequest(String senderID,DecodingResult dec, Query query) {
	// Generate the answer to the query by calling the registry.
	// QueryResults currently simply transport a String. Another option would
	// be to send whole  objects (e.g. Policy)...

	// Here's the place to add a check if authentication is mandatory and is there (in dr).
	// Currently, simple queries can be done by anyone.

	int type=query.getType();
	QueryResult qa=new QueryResult("Unknown query type: "+type);
	Registry registry=node.registry();

	switch(type) {
	    case Query.FREE: {
		qa= new QueryResult((new Integer(registry.freePlaces())).toString()+" out of "+node.MAX_AGENTS() +" agent places are free");break;}
	    case Query.AGENTS_STATE: {
		qa= getAllAgentStates(registry, senderID); break;}
	    case Query.POLICY: {
		// Lookup the policy by asking the registry.
		qa= new QueryResult("The policy for user "+senderID+" is:\n"+registry.authorisationFor(senderID,dec));break;}
	    case Query.NODE_SERVICES: {
		qa= getAllNodeServices(); break;}
	    case Query.NEIGHBORS: {
		qa= getAllNeighbors(); break;}
	    case Query.ROUTING: {
		qa= getRouting(); break;}
	default: {}
	}
	sendMessage(qa);
    }

    protected QueryResult getAllAgentStates(Registry registry, String id) {
	StringBuffer buff=new StringBuffer("Running agents of: ");
	buff.append(id);
	buff.append('\n');
	Vector v=registry.runningAgentsOf(id);
	if(v.size()==0)
	    buff.append("None.");
	else 
	    buff.append(v.toString());
	return new QueryResult(new String(buff));	    
    }

    protected QueryResult getAllNodeServices() {
	StringBuffer buff=new StringBuffer("The node services are: ");
	buff.append(node.nodeServices().toString());
	buff.append('\n');
	return new QueryResult(new String(buff));	    
    }

    protected QueryResult getAllNeighbors() {
	StringBuffer buff=new StringBuffer("The neighbor node IDs are: ");
	buff.append(node.neighborNodes().toString());
	// use neighbors() for debugging.
	buff.append('\n');
	return new QueryResult(new String(buff));	    
    }

    protected QueryResult getRouting() {
	// This is only for testing the routing.
	StringBuffer buff=new StringBuffer("The node routing: ");
	buff.append(node.routing());
	buff.append('\n');
	return new QueryResult(new String(buff));
    }

    protected void sendMessage(Message m) {
	posr.send(new  ProtocolObject(node.NODE_ID(), ProtoConsts.PLAIN ,  ProtoConsts.QUERY_RES,  m));
    }

}
