package node;

import java.util.Date;

public class RessourceUsage {
    // Basically a data container reflecting the currently used ressources of an agent.
    protected ExecutionEnvironment environment;
    protected long startingTime=0;
    protected int serverport=0;


    protected Policy policy; // Duplicated, is also in ExecEnv!

    // These values represent the total usage
    protected long totExecutionTime=0; // in the cpu, not the residing time
    // (can be calced from the startingTime).

    // These values represent current usage (of the last measurement intervall).
    protected long executionTime=0; // Time in the cpu.
    protected long memory=0; // Memory size (not used).
    protected long handledPackets=0; // How many packets are handled normal since the last ressource check.
    protected long emergencyHandle=0; // How many packets are handled chokeing since the last ressource check.
    protected long totHandledPackets=0; //The totals.
    protected long totEmergencyHandle=0; 

    public RessourceUsage(ExecutionEnvironment e, Policy p, int sp) {
	environment=e;
	serverport=sp;
	policy=p;
    }

    public void setStartingTime(long s) {
	startingTime=s;
    }

    public void setMemory(long s) {
	memory=s;
    }

    public synchronized void addExecTime(long t) {
	executionTime+=t;
    }

    public synchronized void incEmergencyHandle() {
	emergencyHandle++;
    }

    public synchronized void incHandledPackets() {
	handledPackets++;

    }

    public synchronized void updateTotal() {
	totExecutionTime+=executionTime;
	totHandledPackets+=handledPackets; 
	totEmergencyHandle+=emergencyHandle;
	resetIntervall(); // to be idempotent
    }

    public synchronized void resetIntervall() {
	executionTime=0; // Time in the cpu.
	//memory=0; // Memory size (not used).
	handledPackets=0; // How many packets are handled normal.
	emergencyHandle=0;
    }

    // --- Getter-methods:

    public ExecutionEnvironment executionEnvironment() {
	return environment;
    }

    public Policy policy() {
	return policy;
    }

    public int serverPort() {
	return serverport;
    }

    public long startingTime() {
	return startingTime;
    }

    public long totalExecutionTime() {
	return totExecutionTime;
    }
    
    public long executionTime() {
	return executionTime;
    }

    public long memory() {
	return memory;
    }

    public long handledPackets() {
	return handledPackets;
    }

    public long emergencyHandled() {
	return emergencyHandle;
    }

    public long handled() {
	// the sum of all treated packets in the last interval.
	return handledPackets+emergencyHandle;
    }


    public long liveTimeMilli() {
	return System.currentTimeMillis()-startingTime;
    }

    public int liveTimeSecs() {
	return (int) liveTimeMilli()/1000;
    }

    public String toString() {
	// Important for the queries, because that is an answer to one (Agents state).
	StringBuffer buf=new StringBuffer();
	buf.append("\n\nRessource usage of Agent: ");
	buf.append(executionEnvironment().agentID());
	buf.append("\n");
	buf.append("Started ");
	buf.append(new Date(startingTime));
	buf.append(" pure running time so far: ");
	buf.append(executionTime);
	buf.append('\n');
	buf.append("Handled packets: ");
	buf.append(totHandledPackets);
	buf.append("  emergency handled: ");
	buf.append(totEmergencyHandle);
	buf.append('\n');
	buf.append(environment.toString());
	return new String(buf);
    }

}
