package clientserver;

import java.io.*;
import java.util.*;

import filter.*;

public class ExecutionRequest extends Message {
    // This class encodes the data to be transmitted during the
    // sending of an agent from client to node.

    protected final char fileSeparator='/'; // To translate classname dots into fileseparators.
    // This is dependent of the file system.
    
    // ---- The data:
    public Vector classNames=new Vector(); // Strings of superclass- and helper class names
    // that need to be loaded. The first element of this vector must be an Agent.
    public Vector classes=new Vector(); // ByteArrays for the bytecode of the named classes.
    public Vector initValues=new Vector(); // This is agent specific.
    public Filter filter; // This is how an agent can request a filter.
    public ForwardingDescriptor forwarding;
    public int ID;// Together with the sender ID this identifies the agent.
    // ---- 

    public ExecutionRequest(String name, Filter f, ForwardingDescriptor d, String path, String extension, int id) {
	ID=id;
	filter=f; // Is usually set afterwards.
	forwarding=d;
	classNames.addElement(name);
	importFromConfigFile(path, extension); // imports the bytecode
	initValues.addElement(name); // The inits should be read from file, too!
	initValues.addElement(new Integer(id));
    }

//     public ExecutionRequest(String name, Vector init) {
// 	// Obsolete!
// 	initValues=init;
// 	classNames.addElement(name);
// 	importFromConfigFile(); // imports the bytecode
// 	filter =new Filter(new All(), new All(), new All(), -1, -1);
// 	// demand as much as you can get.
//     }


    public boolean agentFound() {
	// Returns if the config load was successfull.
	// Should contain other checks, too.
	return (classes.size()>0);
    }

    public String toString() {
	StringBuffer buf=new StringBuffer();
	buf.append(super.toString());
	buf.append("\n Classnames: ");
	buf.append(classNames.toString());
	buf.append("\n Bytecodes: ");
	buf.append(classes.toString());

	return new String(buf);
    }


    public void exportToConfigFile() {
	// Writes a human readable file with the class names.
	// Not used.
    }

    public void importFromConfigFile(String path, String extension) {
	// Initialises from a human readable file.
	// This is used by the client side (gui). The server side uses object deserialisation.
	readClassNames( path,  extension);
	readClasses( path);
    }

    protected void readClassNames(String path, String extension){
	StringBuffer buf=new StringBuffer(128);
	int c=0;
	try {
	    FileReader fr=new FileReader(filename(path,((String) classNames.firstElement()),extension ));
	    c=fr.read();
	    while(c>-1) { // read all characters into buffer
		buf.append((char) c);
		c=fr.read();
	    }
	} catch (IOException e) {System.err.println(e);}
	StringTokenizer parser=new StringTokenizer(new String(buf));
	try {
	    while(parser.hasMoreTokens()) { // put names in the vector
	        treatClassName(parser.nextToken());
	    }
	} catch (NoSuchElementException e) {System.err.println(e);}
    }


    protected void readClasses(String path){
	String n;
	ByteArray b;
	for(Enumeration e=classNames.elements(); e.hasMoreElements(); ) {
	    n=(String) e.nextElement();
	    b=new ByteArray(filename(path, n, ".class")); // Load the class files.
	    if (b.bytes !=null) classes.addElement(b);
	}
    }

    protected void treatClassName(String n) {
	classNames.addElement(n); // some sanity checks would be nice here...
    }

    protected String filename(String pth, String nme, String ext) {
	// constructs the file name of the file that should contain
	// the class named nme.
	int nsize=nme.length();
	StringBuffer buf=new StringBuffer(pth.length()+nsize+ext.length());
	buf.append(pth);
	char c;
	for(int i=0; i<nsize;i++) {
	    c=nme.charAt(i);
	    // Dots in java class names are mapped to file separators '/'.
	    if(c=='.') buf.append(fileSeparator);
	    else buf.append(c);
	}
	buf.append(ext);
	return new String(buf);
    }

    public static void main(String[] args) {
	// testing
// 	ExecutionRequest er= new ExecutionRequest("MyTest", new Vector(1));
// 	System.out.println(er);

    }

}
