package clientserver;

import java.io.*;
import java.net.*;
import java.util.*;

import topology.*;
import node.*;
import utils.*;

public class Forwarder implements Runnable {
    // Forwards (transit) a protocol object (containing an ExecutionRequest)
    // to neighbor nodes.

    protected Node node;
    protected ProtocolObject po;
    protected DecodingResult dr;

    Forwarder(Node n, ProtocolObject p, DecodingResult d) {
	node=n;
	po=p;
	dr=d;
    }
    
    public void run() {
	ForwardingDescriptor fd=po.forwarding;
	if(dr.isAuthentic() ) {  // Only signed agents are forwarded. // deactivate this here.
	    // Alternatively, policy information could be considered.
	    if(fd instanceof HopByHop) {
		hopByHop((HopByHop) fd,po, dr);
		return;
	    }
	    if(fd instanceof Broadcast) {

		broadCast(po, dr, (Broadcast) fd);
		return;
	    }
	} else{
	    System.out.println("No forwarding of unsigned agent of: "+po.senderID);
	}
    }

    void hopByHop(HopByHop fd, ProtocolObject po, DecodingResult dr) {
	System.out.println("HopByHop to: "+fd.finalTarget);
	RouteRecord rr=node.nextNodeTowards(fd.finalTarget);
	if(rr!=null) // If there is routing info.
	    if(rr.hopCount>0) {// If this is not the last node:
		// Send the protocol object to the next hop.
		send(po, node.aboutNode(rr.nextHop));
	    }
    }

    void broadCast(ProtocolObject po, DecodingResult dr, Broadcast descriptor) {
	System.out.println("Broadcast");
	String origin=descriptor.fromID;
	descriptor.fromID=node.NODE_ID(); //Mark us as last hop of the descriptor.
	// Lookup the NodeRecords of all neighbor nodes:
	Vector nodeRecs=node.aboutNodes(node.neighborNodes().neighbors); 
	NodeRecord nr;
	for (Enumeration e=nodeRecs.elements(); e.hasMoreElements();) {
	    // Sending the agent to all neighbors.
	    nr=(NodeRecord) e.nextElement();
	    if(!nr.key.equals(origin)) // Don't loop back to where it came from.
		send(po, nr); 
	}
    }


    protected void send(ProtocolObject po, NodeRecord nr) {
	System.out.println("Forwarding to: "+nr);
	PureClient client= new PureClient(nr.dnsName, nr.port);
	client.openConnection(nr.key); // The receiver node ID.
	ProtocolObjectSenderReceiver posr=client.getProtocolObjectSenderReceiver();
	posr.setEncoder(new PGPEncoder263i(node.PGP_TMP_PATH(), node.NAME()));
	// It follows client protocol behavior:
	posr.transit(po); // Sends the po unchanged (no new encoding).
	ProtocolObject answer;
	DecodingResult dr=posr.receive(node.PASSPHRASE());
	if(dr != null) {
	    answer=dr.protocolObject(); // Ignore the answer.
	    Acknowledgement ack=(Acknowledgement) Message.readObjectFrom(answer);
	    System.out.println("Ignoring: "+ack);
	}
	posr.closeSocket();
    }
}
