package clientserver;

import topology.NodeRecord;
import utils.AddressCalculator;

import java.io.*;
import java.util.*;
import java.net.*;

public class RoutingClient { 
    //Allows the routing table of a node to send routing updates.
    protected Socket socket=null;
    protected int port;
    protected NodeRecord server;
    protected ProtocolObjectSenderReceiver posr=null;

    public RoutingClient(NodeRecord receiver) {
	server=receiver;
    }

 
    public boolean send(RoutingUpdate update, String passPhrase) {
	//System.out.println("Sending update: "+update+" to: "+server);
	port=server.routingPort;
	openConnection(server.key);
	if (socket != null) {
	    ProtocolObject po=new ProtocolObject(server.key, ProtoConsts.PLAIN, ProtoConsts.ROUTE_UPDATE,  update);
	    if(!posr.send(po, passPhrase)) return false; // Sending. return if failed.
	    
	    closeConnection();
	    return true;
	}
	return false;
    }
    
    protected void openConnection(String receiverID) {
	// also generates the protocol object sender/receiver.
	try {
	    // Note we use the IP (not the dns) in the dot-form here. DNS may not
	    // be available in Virtual routers.
	    socket=new Socket(AddressCalculator.toString(server.ip), port);
	} catch (Exception e) {System.err.println(e); socket=null;}
	posr=new ProtocolObjectSenderReceiver(socket, receiverID);
    }

    

    void closeConnection() { // This can also be called by the protocol handler
	if(socket !=null)
	    try {
		socket.close();
	    } catch (IOException e) {
		System.err.println("Client: "+e);
	    }
    }

}
