package clientserver;

import java.io.*;
import java.net.*;

import node.*;
import utils.PGPEncoder263i;

public class ServerPOSenderReceiver extends ProtocolObjectSenderReceiver {
    // Decodes the protoObj and starts the appropriate Handlers
    // Specialized to demultiplex Query resp. ExecutionRequest.
    // for further processing.

    protected Node node; // holds together the different node classes.
    protected byte encoding;// Stores the encoding that the client uses in order to answer in the same way.
    protected Thread forwarding=null; // Forwards agents to neighbors if requested.

    public ServerPOSenderReceiver(Socket s, Node n) {
	super(s, ""); // the receiver ID is only known after a request.
	node=n;
	pgp=new PGPEncoder263i(node.PGP_TMP_PATH(), n.NODE_ID()); // To run the client and the server on the
	// same file system, the tmp paths must be different.
    }

    public boolean finished() {
	// Is the underlying connection closed or broken?
	return finished;
    }

    public void setEncoding(byte e, String recID) {
	encoding=e;
	receiverID=recID;
    }


    public  boolean send(ProtocolObject object) {
	object.encoding=encoding; // Answer in the same way the client asked.	
	// Set encoding=ProtoConsts.PLAIN herefor testing the client warning.
	if(!super.send(object,node.PASSPHRASE())) {
	    finished=true;
	    return false;
	}
	return true;
    }


    public boolean send(Result r) { // was Result..
	 // Method for convenient access.
	return send(new ProtocolObject(node.NODE_ID(), ProtoConsts.PLAIN ,  ProtoConsts.RESULT,  r));
     }

    public boolean send(NumericResults r) { // was Result..
	 // Method for convenient access.
	return send(new ProtocolObject(node.NODE_ID(), ProtoConsts.PLAIN ,  ProtoConsts.RESULT,  r));
     }

    public synchronized DecodingResult receive() {
	DecodingResult dr=super.receive(node.PASSPHRASE());
	if(dr != null) {
	    ProtocolObject po=dr.protocolObject();
	    receiverID=po.senderID; // Store the sender ID, since this will be
	    // the receiverID when the server answers. 
	    encoding=po.encoding;
	    startTypeSpecificHandlers(dr);
	    return dr; // This is a little strange, since the value will not be used.
	}
	else {
	    // Normally the client closes the connection. But since the client
	    // sent garbage, the server does it here:
	    closeSocket();
	    return null;
	}
    }


    protected void startTypeSpecificHandlers (DecodingResult o) {
	switch (o.protocolObject().messageType) {
	case ProtoConsts.EXEC_REQ: {
	    startExecRequestHandler(o);
	    break;
	}
	case ProtoConsts.QUERY: {
	    startQueryHandler(o);
	    break;
	}
	default: {
	    error=new ErrorMessage("ProtocolObject demands unimplemented handler.");
	}
	}
    }

    protected void startExecRequestHandler (DecodingResult dr) {
	ProtocolObject po=dr.protocolObject();
	Object o=Message.readObjectFrom(po);
	if(o instanceof ExecutionRequest) {
	    ExecutionRequest r=(ExecutionRequest) o;
	    poFromWire.forwarding.updateHopCount(); // Now the agent has hopped once more.
	    r.forwarding=po.forwarding;// Forwarding is unfortunatelly redundant..
	    if(poFromWire.forwarding instanceof Forwarding){ // Is there some forarding at all
		// Prepare the handling of forwarding if necessary.
		forwarding=new Thread(new Forwarder(node, poFromWire , dr)); // started by
		// a callback from the execRequ handler.
		// poFromWire is the proto obj as received in the super class.
	    } 
	    // Now start the agent here:
	    ExecutionRequestHandler handler=
		new ExecutionRequestHandler(node, po.senderID, dr , r, this);
	}
    }

    public void startForwarding() {
	// If the agent started well here, forward the agent (called by ExecRequestHandler).
	if (forwarding!=null) forwarding.start();
    }

    protected void startQueryHandler (DecodingResult o) {
	// ignoring if auth or not.. add stuff here!
	ProtocolObject po=o.protocolObject();
	Query q=(Query)
	    Message.readObjectFrom(po);
	QueryHandler handler=
	    new QueryHandler(node,po.senderID, o, q, this);
    }

}
