package config;

import java.io.*;
import java.util.*;

public class NodeConfigManager extends ConfigObjectManager {
    // Manages (loads/saves) the node configuration database 
    protected static final String separator=";"; // For the file format.
    protected static final String whitespace="\t"; 

    protected String configFileName;// including path and extension.
   
    protected StringTokenizer parser;


    public NodeConfigManager(String filename) {
	configFileName=filename;
    }

    public ConfigObject defaultConfig() {
	return readFromDisk(configFileName);
    }



    // File handling



    public void writeToDisk(String name, ConfigObject o) {
	BufferedWriter writer;
	String sep=whitespace+separator+whitespace;
	String conf;
	
	try {
	    writer=new BufferedWriter(new FileWriter(name));
	    for(Enumeration e=o.keys(); e.hasMoreElements(); ) {
		conf=(String) e.nextElement();
		writer.write(conf); // write the key.
		writer.write(sep);
		conf=(String) o.get(conf); 
		writer.write(conf); // write the conf value.
		writer.write("\n");
		writer.flush(); // necessary.
	    }	    
	} catch (Exception e) {
	    System.err.println("NodeConfigManager couldn't write "+o+" in text form");
	}
    }

    public ConfigObject readFromDisk(String name) {
	ConfigObject ret=new ConfigObject();
	BufferedReader reader;
	String line;

	try {
	    reader=new BufferedReader(new FileReader(name));
	    line=reader.readLine();
	    while(line != null) {
		readLine(ret,line);
		line=reader.readLine();
	    }
	} catch (Exception e) {
	    System.err.println(e+"\nNodeConfigManager couldn't read configs from "+name);
	}
	return ret;
    }

    protected void readLine(ConfigObject co, String line) {
	parser=new StringTokenizer(line,separator);
	String key=giveNextToken(parser);
	if(key==null) return; // skip empty line
	String value=giveNextToken(parser);
	if(value==null) {
	    System.err.println("NodeConfigManager error: no value provided for: "+key);
	    System.exit(8);
	}
	co.put(key, value); // Enter the parsed value into the config object.
	value=giveNextToken(parser);
	if(value != null) {
	    System.out.println("NodeConfigManager warning: unused token: "+value);
	}
    }

    protected String giveNextToken(StringTokenizer parser) {
	String ret=null;
	try {
	    ret=parser.nextToken();
	    ret=ret.trim(); // eliminate leading/trailing whitespaces.
	} catch(Exception e) {}
	return ret;
    }


    public static void main(String[] args) {
	NodeConfigManager m=new NodeConfigManager("/home/mguenter/rvs/Agents/config/nodeConfig.txt");
	ConfigObject co=m.defaultConfig();
	//m.writeToDisk(n.USER_DB_FILENAME(), m.manuallyCreate());	 // bootstrap

	System.out.println(co);
	m.writeToDisk("/home/mguenter/rvs/Agents/config/nodeConfig.txt",co );
    }
}
