package config;

import java.io.*;
import java.util.*;

import node.*;
import filter.*;
import utils.AddressCalculator;

public class UserDBManager extends ConfigObjectManager {
    // Manages (loads/saves) the user database (user ID- user profile).
    // Two persistent forms can be used: object serialisation and a
    // human readable version. For the later see also 'ProfileParser'.
    // node.Registry uses the defaultConfig() method to get its database.

    public String readableTextExtension;
    public String objExtension;

    protected Node node;
    protected ProfileParser profileParser;

    public UserDBManager(Node n) {
	node=n;
	init(n.CONFIG_PATH());
	profileParser=new ProfileParser(node);
    }

    public ConfigObject defaultConfig() {
	//return manuallyCreate();
	return readFromDisk(node.USER_DB_FILENAME());
    }

    // File handling


    public ConfigObject readFromDisk(String pureName) {
	if(node.USER_DB_AS_OBJECT()) {
	    return readObjFromDisk(appendExtension(pureName));
	}
	return readTxtFromDisk(createCompleteFileName(pureName));
    }

     public void writeToDisk(String pureName, ConfigObject o) {
	if(node.USER_DB_AS_OBJECT()) {
	    writeObjToDisk(appendExtension(pureName),o);
	} else {
	    writeTxtToDisk(createCompleteFileName(pureName),o);
	}
    }

  
//     protected ConfigObject manuallyCreate() {
// 	// Initialise the db for testing or bootstrapping purposes.
// 	ConfigObject userDB=new ConfigObject();
	
// 	SuperPolicyGen spg= new SuperPolicyGen(node);
// 	CustomerPolicyGen cpg= new CustomerPolicyGen(node);
// 	TestPolicyGen tpg= new TestPolicyGen(node);

// 	// Define me as a user who administers the uni network.
// 	Range iam=AddressCalculator.parseRange("130.92"); 
// 	Range cui=AddressCalculator.parseRange("129.194.90"); 
// 	Range priv=AddressCalculator.parseRange("172"); // maybe private adds
// 	// should be accessible to all agents?
// 	ListOfRanges homeNets=new ListOfRanges(iam);
// 	homeNets.addRange(cui); homeNets.addRange(priv);

// 	String id="mguenter@iam.unibe.ch";
// 	Profile me=new Profile(id, homeNets, spg); // Ev. profiles should have several
// 	// user id per policy generator!
// 	userDB.put(id, me);

// 	id="cui.unige.ch";
// 	homeNets=new ListOfRanges(cui);
// 	Profile cuiprofile=new Profile(id, homeNets, cpg); 
// 	userDB.put(id, cuiprofile);


// 	// Define an anonymous user with no home network
// 	homeNets=new ListOfRanges(); //(empty list) is not administrator of any network.
// 	id="anonymous";
// 	Profile anonym=new Profile(id,homeNets, tpg);
// 	userDB.put(id, anonym);
// 	return userDB;
//     }

    // File handling helpers

    protected void writeObjToDisk(String nameNext, ConfigObject o) {
	saveToDisk( nameNext, o);
    }

    protected ConfigObject readObjFromDisk(String nameNext ) {
	return loadFromDisk(nameNext);
    }

    protected void writeTxtToDisk(String name, ConfigObject o) {
	try {
	    profileParser.setWriter(new FileWriter(name));
	    Profile p;
	    for(Enumeration e=o.elements(); e.hasMoreElements(); ) {
		p=(Profile) e.nextElement();
		profileParser.write(p);
	    }	    
	} catch (Exception e) {
	    System.err.println("UserDBManager couldn't write "+o+" in text form");
	}
    }

    protected ConfigObject readTxtFromDisk(String name) {

	ConfigObject ret=new ConfigObject();
	try {
	    profileParser.setReader(new FileReader(name));
	    Profile p;

	    while((p=profileParser.read()) != null) {
		ret.put(p.identity(), p);
	    }
	} catch (Exception e) {
	    System.err.println(e+"\nUserDBManager couldn't read object as text from "+name);
	}
	return ret;
    }

    protected String createCompleteFileName(String pureName) {
	return appendExtension(diskPath+pureName);
    }

    protected String appendExtension(String to) {
	String extension= node.USER_DB_AS_OBJECT()? node.USER_DB_OBJ_EXT() : node.USER_DB_TXT_EXT() ;
	return to+extension;
    }

    public static void main(String[] args) {
	Node n=new Node("/home/mguenter/rvs/Agents/config/nodeConfig.txt");// dummy

	UserDBManager m=new UserDBManager(n);
	ConfigObject co=m.defaultConfig();
	//m.writeToDisk(n.USER_DB_FILENAME(), m.manuallyCreate());	 // bootstrap

	System.out.println(co);
	m.writeToDisk(n.USER_DB_FILENAME(), co);
    }
}
