package netgui;

/* Written by Manuel Gnter. All rights reserved, no warranties */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;


public class NetCanvas extends Component implements MouseListener, MouseMotionListener {
    protected Vector domains=new Vector(); // contains also EndUser
    protected int posCntr=0;

    protected NetComponent selected=null;
    protected boolean dragging=false;

    protected Image offImage; // Vars for double-buffering (avoids glitches).
    protected Graphics offGraphics=null;
    protected Dimension offDimension=new Dimension(0,0);

    public void update(Graphics g) {
	Dimension currentSize=getSize();
	// Create new double buffer if size has changed:
	if ((offGraphics==null) || (!offDimension.equals(currentSize))) {
	    offDimension=currentSize;
	    offImage=createImage(currentSize.width, currentSize.height);
	    offGraphics=offImage.getGraphics();
	}
	
	offGraphics.setColor(getBackground()); // To the old picture.
	offGraphics.fillRect(0,0,currentSize.width, currentSize.height);

	NetComponent d; // Draw all net components       
	for(Enumeration e=domains.elements(); e.hasMoreElements(); ) {
	    d=(NetComponent) e.nextElement();
	    d.paintOn(offGraphics);
	}

	g.drawImage(offImage,0,0,this);

    }

    public void paint(Graphics g) {
	update(g);
    }

    public void repaint() {
	// Must also be overriden, else there is still flashing. Side effects?
	update(getGraphics());
    }

    public void mouseClicked(MouseEvent me) {
	if(checkIfSelects(me.getX(),me.getY()))
	    repaint();
    }
    public void mousePressed(MouseEvent e) {}
    public void mouseReleased(MouseEvent me) {
	if (dragging) {
	    ((RellocNetComponent) selected).move(me.getX(),me.getY());
	    dragging=false;
	}
	repaint();
    }
    public void mouseEntered(MouseEvent e) { }
    public void mouseExited(MouseEvent e) {}
    public void mouseMoved(MouseEvent e) {}
             

    public void mouseDragged(MouseEvent me) {
      if (!dragging)
	  if(checkIfSelects(me.getX(),me.getY())) {
	      dragging=true;
	  }
      if (dragging) {
	  ((RellocNetComponent) selected).move(me.getX(),me.getY());
	  repaint();
      }
    }

    protected boolean checkIfSelects( int x, int y) {
	NetComponent d;
	for(Enumeration e=domains.elements(); e.hasMoreElements(); ) {
	    d=(NetComponent) e.nextElement();
	    d=d.touching(x,y);
	    if (d!=null){
		if(selected != null) {
		    addLinkIfRequested(d);
		    selected.deselect();
		}
		selected=d;
		d.select();
		return true;
	    }
	}	
	return false;
    }

    public Dimension getPreferredSize() { 

 	return new Dimension(400,400);
    }

    public void addNewDomain() {
	domains.addElement(new Domain(posCntr++,0));
	repaint();
    }

    public void addNewRouter() {
	Router router;
	if(selected!=null) {
	    if(selected instanceof Domain) {
		Domain parent=(Domain) selected;
		router=new Router(parent, this);
		parent.addRouter(router);
		router.attach(new ValueDisplayer(0,1.0));
		repaint();
	    }
	}
    }

    public void addNewEndUser() {
	domains.addElement(new EndUser(this,posCntr++));
	repaint();
    }

    protected void addLinkIfRequested(NetComponent newSel) {
	if(newSel instanceof Router)
	    if (selected instanceof Router) {
		Router a=(Router) newSel;
		Router b=(Router) selected;
		if (a.domain() != b.domain()) {
		    Link l=new Link(a,b);
		    a.addLink(l);
		    b.addLink(l);
		}
	    }
    }    
}
