package node;

import capsule.Agent;
import clientserver.Result;
import utils.ObjectQueue;
import java.util.*;
import java.io.*;

public class RessourceController implements  Runnable {
    // Checks periodically weather all agents stick to the rules.
    // Some of the 'rules' are set via the policy of the agents,
    // some are set in the constructor.
    // Started by the Registry.

    protected long lastCheckTime;
    protected long now;

    protected long intervall; // Period of the checks.

    protected double intolerableLoss;
    protected long maxBurstLen;
    protected long minSignificantExecLen;
    protected int maxTime2normal;
    protected double memoryCheckProbability=0.3; // The mem check is expensive-> don't do it all the time. Maybe this should be set in Node.

    protected Registry registry; // For killing an agent.

    protected ExecutionEnvironment currentEE; // Tmp vars to reduce method calls.
    protected ObjectQueue queue;
    protected Policy policy;

    protected double currentWorstRating; // 0..1 , the smaller the worse.
    protected RessourceUsage worst;
    protected short lowestPriority=127; // stores the priority of the 'worst'.
    protected int time2normal=0;

    public RessourceController(Registry reg, long interv, long mabu, double lossPerc, long minsig, int maxnorm) {
	registry=reg;
 	intolerableLoss=lossPerc;

	maxBurstLen=mabu;
	minSignificantExecLen=minsig;
	maxTime2normal=maxnorm;

	lastCheckTime=System.currentTimeMillis();
	intervall=interv;
    }

    public void run() {
	Thread.currentThread().setPriority(Thread.MAX_PRIORITY);
	// Higest priority is crucial to preempt agents and to keep consistency.
	while(true) {
	    now=System.currentTimeMillis();
	    checkUsages();
	    lastCheckTime=now;
	    try {Thread.sleep(intervall);}catch(InterruptedException e){};
	}
    }

    // // Old test approach: was infeasible
//     protected void checkUsages() {
// 	// The test loop through the currently running agents.
// 	System.out.println("?? Check");
// 	RessourceUsage waster=new RessourceUsage(null,null, 0); //to find out an agent to delete.
// 	long wasterCPU=0;
// 	long cpuTime=0;
// 	double totalTime=now-lastCheckTime;
// 	boolean unacceptable=false;
// 	//boolean badwaster=false;

	
// 	Enumeration e=registry.getRunningAgents();
// 	interleav=new InterleavingResolver(e, now, (long) (burstLenPercentage*intervall));
// 	System.out.println(interleav);
// 	//interleav.ajustFinalBursts(); //


// 	for(e=registry.getRunningAgents(); e.hasMoreElements(); ) {
// 	    RessourceUsage ru=(RessourceUsage) e.nextElement();
// 	    currentEE=ru.executionEnvironment(); // Used by calculateCPUTime and queueFull.
// 	    cpuTime=calculateCPUTime(ru);
// 	    System.out.println(currentEE.agent()+" consumes (CPU): "+cpuTime+" handled: "+ru.handledPackets()+" emerg: "+ru.emergencyHandled());
// 	    if (cpuTime>=wasterCPU) {
// 		waster=ru;
// 		wasterCPU=cpuTime;
// 		// Maybe something like priority should also be considered.
// 	    }
// 	    if(queueFull()) {
// 		unacceptable=true;
// 	    }
// 	    if(!unacceptable) 
// 		unacceptable=testAcceptablility(ru);
// 	    ru.updateTotal();
// 	    checkTimes(ru);
// 	}
// 	System.out.println("Resumee: totalTime  "+totalTime+ " wasterCPU  "+wasterCPU);
// 	if ((totalTime-wasterCPU) <= minimumNodeUsage*totalTime) {
// 	    registry.setCongestion(true);
// 	    System.err.println("Agent congests the node!");
// 	    registry.stopAgent(waster.executionEnvironment().agent(), new Result("Agent is stopped because:\nAgent uses too much CPU!"));
// 	} else if (unacceptable){ // At least one agent gets bad service.
// 	    registry.setCongestion(true);
// 	    //  if(wasterCPU*registry.nrOfAgents()>=intolerableOverusage*totalTime) {// Check if the waster uses signif. more than average. ->Only bad wasters get killed.
// 	    System.err.println("Bad CPU waster agent found!");
// 	    registry.stopAgent(waster.executionEnvironment().agent(),new Result("Agent is stopped because:\nAgent is unfair to other agents!"));
// 		// }
// 	} else {
// 	    registry.setCongestion(false);
// 	}
//    }

    protected void checkUsages() {
	// The test loop through the currently running agents.

	//System.out.println("?? Check");
	double totalTime=now-lastCheckTime;
	boolean unacceptable=false; // Indicates if there is a congestion.
	
	Enumeration e=registry.getRunningAgents();
	initGlobalVals(); // for checkIfWorse()
	for(e=registry.getRunningAgents(); e.hasMoreElements(); ) {
	    RessourceUsage ru=(RessourceUsage) e.nextElement(); // Set temp vars.
	    currentEE=ru.executionEnvironment(); // (minimize method calls)
	    queue=currentEE.agentWrapper().queue();
	    policy=ru.policy();
	  
	    burstAndCPUchecks(ru); // Determines also who makes the least out of its cpu time.
	    checkSize(ru); // Memory check, only works partially.
	    
	    if(queueFull()) {  // Are the agents fast enough?
		unacceptable=true;
	    }
	    if(!unacceptable) 
		unacceptable=testAcceptablility(ru); // Packet loss rate.
	    ru.updateTotal();
	    checkTotalTimes(ru); // TTL, total exec time etc.
	}
	 
	if (unacceptable) { // At least one agent gets bad service.
	    registry.setCongestion(true); 
	    if((time2normal<=0) && (worst !=null)) { // We have a congestion, a waster and haven't done something about it already.
		killAgent(worst.executionEnvironment(), "Agent is stopped because:\nAgent does not make fair usage of the CPU!\nRating: "+currentWorstRating);
	    }
	} else 
	    registry.setCongestion(false);
    }

    protected void burstAndCPUchecks(RessourceUsage ru) {
	long lastburst=calculateLastBurst();
	long cpuTime=calculateCPUTime(ru,lastburst);
	//System.out.println(currentEE.agent()+" consumes (CPU): "+cpuTime+" handled: "+ru.handledPackets()+" emerg: "+ru.emergencyHandled());

	if(cpuTime>=minSignificantExecLen) { // Only check agents that are executing for some time.
	    if(lastburst>maxBurstLen) { // Detects agent routines that don't stop at all.
		killAgent(currentEE,"Agent is stopped because:\nAgent had a too long CPU burst of: "+cpuTime);
	    } else
		checkIfWorse(ru,cpuTime); // Sets the kill candidate 'worst'.
	}
    }

    protected void checkSize(RessourceUsage ru) {
	// This does only work on serialisable agents :-(
	// It is a heavy operation.
	if(Math.random()>=memoryCheckProbability) return; // Only do it sometimes.
	Agent agent=currentEE.agent();
	ByteArrayOutputStream baOut=new ByteArrayOutputStream();
	long size=0;
	agent.putServiceChannel(null); // Risky, but we don't want to serialize the ResultChannel
	//long strt=System.currentTimeMillis();
	try {
	    ObjectOutputStream out=new ObjectOutputStream(baOut);
	    out.writeObject(agent);
	    size=baOut.toByteArray().length;
	} catch (Exception e) {
	    //System.err.println("Could not serialize agent: "+agent+" error: "+e);
	    // If this was a node requirement, we could kill the agent now.
	}
	agent.putServiceChannel(currentEE.serviceChannel()); // Gives the channel back.
	//long end=System.currentTimeMillis();
	//System.out.println("SIZE of agent: "+agent+" : "+size+" Mesrtime: "+(end-strt));
	if(size>policy.maxMem()) {
	    killAgent(currentEE, "Agent is stopped because:\nAgent uses too much memory: "+size);
	}
    }


    protected void checkTotalTimes(RessourceUsage ru) {
	// Checks the ttl and the total exec time. 
	if (ru.totalExecutionTime()>policy.maxExecTime()) {
	    killAgent(currentEE, "Agent is stopped because:\nAgent consumed all its CPU time.");
	} else if(ru.liveTimeSecs()>=policy.time2live()) {
	    killAgent(currentEE, "Agent is stopped because:\nAgent livetime has expired.");

	}
    }

    protected void killAgent(ExecutionEnvironment ev, String message) {
	// Kills the agent and sets time2normal to give a chance for
	// the congestion to disolve.
	Agent agent=ev.agent();
	System.out.println("Agent killed: "+agent);
	System.err.println(message);
	registry.stopAgent(agent ,new Result(message, true));
	time2normal=maxTime2normal;
    }

    protected void initGlobalVals() {
	currentWorstRating=1.0; // 0..1 , the smaller the worse.
        RessourceUsage worst=null;
	time2normal--;
    }

    protected void checkIfWorse(RessourceUsage ru,long cpuTime){
	// Implements the mechanism to select an agent to kill in case of congestion.
	short curPriority=policy.priority();
	if(curPriority>lowestPriority) return; // Only equ or smaller priorities
	    // are possible kill candidates.

	long handled=ru.handled();
	double newlyArrived=queue.changedSinceLast()+handled;

	if(newlyArrived==0) {
	    if(handled != 0)
		System.err.println("Inconsistency in RessourceController: 0 packet arrived, but "+handled+" handled");
	    handled=1;
	    newlyArrived=1;// No packet had to be treated, but there was some
		//computation (e.g.services). We treat this as if one packet was handled.
	}
	double rating=handled/(newlyArrived*cpuTime); // The formula to
	// calculate the cpu effectiveness.



	if(curPriority<lowestPriority) {
	    worst=ru;
	    currentWorstRating=rating;
	    lowestPriority=curPriority;
	    return;
	}
	// If both priorities are equal:


	if(rating< currentWorstRating) {
	    worst=ru;
	    currentWorstRating=rating;
	}
    }

    protected long calculateLastBurst() {
	// Ongoing bursts are not accounted yet, so we take them into consideration now.
	AgentWrapper aw=currentEE.agentWrapper();
	if(aw.running()) { // Is there an ongoing burst?
	    return now-aw.startTime(); // Account for the time consumed.
	}
	return 0;
    }

    protected long calculateCPUTime(RessourceUsage ru, long lastburst) {
	return ru.executionTime()+lastburst;
    }

    protected boolean queueFull() {
	// Indicates a congestion. 
	//System.out.println(currentEE.agent()+" q-free: "+queue.free());
	return (queue.free()<1);
    }

    protected boolean testAcceptablility(RessourceUsage ru) {
	// The acceptability is tested by comparing how many packets
	// could be treated regularly and how many urgently.
	// Asking the agent here would also be a possibility.
	long lost=ru.emergencyHandled();
	long tot=lost+ru.handledPackets();
	if(tot==0) return false; // The agent was probably idle.
	double lossPerc=lost/((double) tot);
	//System.out.println(" lossPercent "+lossPerc);
	return (intolerableLoss <= lossPerc);
    }




}

