package node;

import clientserver.*;
import capsule.Agent;
import utils.ObjectQueue;

import java.net.*;
import java.io.*;

public class ResultChannelV1 extends ResultChannel {
    // Encapsulates the ProtocolObjectSenderReceiver which is connected to the home-app;
    // Could contain traffic restriction mechanisms.
    // In order to avoid security execptions the sending mechanism takes place using
    // a queue.
    protected ServerPOSenderReceiver posr;
    protected ObjectQueue queue; // Stores 'Result' objects to be forwarded.
    protected boolean stop=false; // False the net-connetion is still open.
    protected boolean once=false; // Ensures that the ResultChannel is run only once.
    //  protected boolean comFinished=false;// When the agent puts a last result in the queue. // the last-result will arrange that the client will close the connection 
    protected boolean comBroken=false;// When the connection is gone (e.g. homeApp has stopped).

    protected boolean informRegistry=true; // Inform the registry after stopping.

    public ResultChannelV1(Agent agent, Node node, ServerPOSenderReceiver s) {
	init(node,agent);
	posr=s;
	queue=new ObjectQueue(node.Q_LEN()); //node.Q_LEN(); 
	// 1 makes sure, that all results are sent immediately.
    }


    public void send(Result r) {
	queue.put(r);
    }

    public boolean connectionBroken() {
	// The agentwrapper can see, that there is use  to send final results
	return comBroken;
    }

    public void send(String s, boolean last) {
	// Encapsulates the string in a reply object (other data might be handy, too).
	// The boolean indicates if this is the last result provided by the agent.
	Result r=new Result(s, last);
	send(r);
    }

//     synchronized boolean stopped() {
// 	return stop;
//     }
	
     synchronized void stop(boolean informRegistry) { 
	 // Overrides the super method: we don't want to stop the thread since
	 // the final results should be sent first.
	this.informRegistry=informRegistry;
	stop=true; 
	// posr.closeSocket(); // The client should close the socket.
    }

    ObjectQueue getQueue() {
	// For the AgentWrapper, to but a ResultDummy.
	return queue;
    }

    public void run() {
	// Read the results from the queue and send them
	if (once) return;
	once=true;
	Result r;
	while(true) { // The sending loop. Stopped by the registry (stop var) or by a sending error.
	    r=getNext();
	    if(stop) {
		sendRest(r);
		break;
	    }
	    if(!posr.send(composeProtoObj(r))) {
		System.err.println("ResultChannel: Sending failed for result: "+r);
		comBroken=true;
		break;
	    }
	}
	if(informRegistry) node.registry().stopAgent(agent, new Result("Agent stopped because:\nconnection to home application is broken.")); // Deregistering of the agent.   //This must not necessarily be the case! Maybe the agent lives longer than the connection to the home app.
	System.out.println("Result Channel closed");
    }

    protected void sendRest(Result r) {
	// After the in-queue stopped, there may be still some results to go out.
	int rest=queue.used();
	for(int i=1; i<=rest; i++) {
	    if(!posr.send(composeProtoObj(r))) { 
		System.err.println("ResultChannel: Couldn't send the last "+(rest-i)+" Results!");
		break;
	    }
	    r=getNext();
	}
	if (! (r instanceof DummyResult)) // AgentWrapper.stop() puts a dummy in the queue to deblock.
	    System.err.println("ResultChannel: Last result is not a dummy as expected but: "+r);

    }

    protected  Result getNext() {
	Result ret=(Result) queue.get();
	//if(r.isLast()) comFinished=true;
	
	return ret;
    }

    protected ProtocolObject composeProtoObj(Result r) {
	return new ProtocolObject(node.NODE_ID(), ProtoConsts.PLAIN ,  ProtoConsts.RESULT,  r);
    }
}
