package node;

import capsule.Agent;
import utils.ObjectQueue;
import node.services.*;
import clientserver.*;
import java.util.*;

public class ServiceChannelV1 extends ServiceChannel {
    protected ServiceList serviceList=new ServiceList(new Hashtable());

    protected boolean once=false; // Ensures that the ResultChannel is run only once.
    protected boolean stopping=false;
    protected Result finalMessage=null;

    public ServiceChannelV1(Agent agent, Node node) {
	init(node,agent);
	queue=new ObjectQueue(node.Q_LEN()); //node.Q_LEN(); 
    }


    void initServiceList(Vector serviceNrs, ExecutionEnvironment ev) {
	// Must be called to init the list, else it is empty -> no node services.
	serviceList=node.nodeServices().services(serviceNrs, node, ev); 
    }

    public void request(Service s) {
	queue.put(s);
    }

    void finalMessage(Result f) {
	finalMessage=f;
    }

	
//      synchronized void stop(boolean informRegistry) { 
// 	 // Overrides the super method: we don't want to stop the thread since
// 	 // the final results should be sent first.
// 	this.informRegistry=informRegistry;
// 	stopping=true; 
// 	// posr.closeSocket(); // The client should close the socket.
//     }


    public void run() {
	if (once) return;
	once=true;
	// Read the results from the queue and send them
	Service s;
	while(!stopping) { // The handler loop
	    s=getNext();
	    handleService(s);
	}
	emptyQueue(); //  necessary?
	handleFinalMessage(); // was one line later -> error.
	serviceList.stopAllHandlers();
 
	System.out.println("Service Channel closed");
    }

    protected void handleFinalMessage() {
	// For error messages of the node, eg. ressource control issues.
	System.out.println("Final message: "+finalMessage);
	if(finalMessage!=null) {
	    handleService(new Service(ServiceConsts.SEND(), finalMessage));
	}
    }

    synchronized void stop(boolean informRegistry) {
	// Override default. The thread is not stopped but rolls out,
	// in order to be able to deliver a final message.
	this.informRegistry=informRegistry;
	stopping=true;
	if (Thread.currentThread()!=myThread) {
	    // make sure the channel does not block.
	    if(queue.empty()) {
		queue.put(new Service(ServiceConsts.STOP_ME(),null));
	    }
	}
    }



    protected void emptyQueue() {
	// Empties the queue
	int size=queue.used();
	for(int i=0; i< size; i++) {
	    getNext();
	}
	System.out.println("ServiceChannels throws "+size+" requests away.");
    }

//     protected void initBasicServices(ServerPOSenderReceiver posr) {
// 	if (serviceList==null) {
// 	    serviceList=new ServiceList(new Hashtable());
// 	}
// 	ServiceHandler resChan=new HomeChannel(posr,  node);
// 	serviceList.addService(9000000,resChan, "Sending on connection of agent arrival.");
//     }



    protected void handleService(Service s) {
	if(s.type()==0) {
	    if(!stopping) {
		node.registry().stopAgent(agent,null); // stopping service. For simplicity directly built-in.
	    }
	} else {
	    startClock(); // Account the service time to the agent.
	    boolean ok=serviceList.startService(s); // Execute the service.
	    stopClock();
	    if(!ok) { // We stop the agent if something was wrong. (->Alt: error sending)
		if(!stopping) { // An error occured, and we are not already stopping.
		    node.registry().stopAgent(agent, new Result("Agent stopped because:\n Service failed: "+s)); 
		    stopping=true;
		}
	    }
	}
    }

    protected Service getNext() {
	Service ret=(Service) queue.get();
		return ret;
    }

}
