package node;

import filter.*;
import utils.*;
import clientserver.ProtoConsts;
import java.net.*;
import java.io.*;


public class T_Configurator implements Runnable {
    // Starts the listening component (T-Component) by sending
    // executing a script that sends a rsh command.

    protected static String init="sh ";
    protected String t_location; // ="wind.cnds.unibe.ch"; //fermat 

    protected StringBuffer script=  new StringBuffer(init);
    protected String vrRequest;

    protected boolean local=false;  // executing the script locally.
    protected boolean fromFile=false; 
    protected boolean real; // performing a real tcpdump;
    protected boolean byScript=false;  // Is the t-component started by a script.
    protected boolean byVR=false;  // Is the t-component in a virtual router.
    protected boolean connectedToVR=false;  // Indicates if a vr connection was established.



    protected String testFileName; // ="logDump.txt";//="ahEsp3000x1024.tcpdump.output"; // ="ahEspShort.txt";//="test.txt";//  ="ahEspShort.txt"
    protected String server; // For setting the 'return address' in the scripts.
    protected int serverPort;
    protected Filter filter;
    protected Node node;
    protected Receiver receiver;

    protected T_ComponentDummy t_dummy=null;

    
    public T_Configurator(Node n, int p, Filter f, Receiver r) {
	node=n;
	serverPort=p;
	filter=f;
	receiver=r;
	server=node.localHost();
	t_location=node.T_NAME(); // Either one or the other is used (dep on 'fromFile').

	if(t_location.equals("")) local=true;  // Set flags.
	testFileName=node.TEST_FILE_NAME();
	if(!testFileName.equals("")) {
	    fromFile=true; // It's a live test when no testfile is declared.
	    byScript=true;
	} else byVR=node.T_IS_VIRTUAL(); // Testfile overrides vr
	if(!byVR && !fromFile && !local) {
	    real=true; // Only remote real sniffing is allowed.
	    byScript=true;
	} 		
	//System.out.println("local "+local+ " fromFile "+fromFile+" byVR "+byVR+" real "+real+" byScript "+byScript);

	initTcomponents() ;// if vr are used, init is different
    }
    
    public void vrConnect() {
	// must be called immediately before the receiver is started.
	if(byVR) {
	    connectToVR();	    
	}
    }

    protected void initTcomponents() {
	// & also init the receiver.
	if (byScript) {
	    createScript(node.SCRIPT_PATH());
	    return;
	} else if(byVR) {
	    initForVR();
	    return;
	}
	t_dummy=new T_ComponentDummy(node.T_DUMMY_INTERVAL(), server, serverPort );
    }

   

    protected void initForVR() {
	// Sets the request string.
	VRFilterStringGenerator gen= new VRFilterStringGenerator();
	vrRequest=gen.filterString(filter);
	System.out.println("VRFilterStringGenerator: "+vrRequest);
    }

    protected void connectToVR() {
	// Opens a connection to the virtual router and orders traffic.
	int port=node.SERVER_PORT()+ProtoConsts.VR_PORT_OFFSET;
	// the port of the t-bone must be serverport+offset (convention).
	System.out.println("Connecting to VR: "+t_location+":"+port);
	Socket socket=null;
	try{
	    socket=new Socket(t_location,port);
	    PrintWriter writer=new PrintWriter(socket.getOutputStream());
	    writer.print(vrRequest);
	    writer.flush();
	    connectedToVR=true;

	} catch (IOException e) {
	    System.err.println("Connection to VR t-bone failed: "+e);
	}

	receiver.setSocket(socket);	// Maybe only if things went well

    }

    public void run() {
	if(byScript)
	    SkriptExec.executeScript(new String(script), false);
	else if (t_dummy!=null) {
	    startDummyTcomponent();
	} else {
	    if(!connectedToVR) 
		System.err.println("Warning: T_Configurator.vrConnect() was not called or not successful!");
	}
    }

    protected void createScript(String path) {
	script.append(path); 
	if(!fromFile && !local) {
	    createRemoteLiveScript();
	} else {
	    appendScriptName();
	}
	appendFirstArg();
	appendRestArgs();
	//System.out.println("T_Configurator uses script: "+script);
    }


    protected void appendScriptName() {
	script.append("start_T_");
	if(local) script.append("local_");
	else  script.append("remote_");
	if(fromFile)  script.append("file ");
	else  script.append("live "); // prob. obsolete
    }

    protected void appendFirstArg() {
	if(!fromFile) {
	    script.append("\'");
	    script.append(filter.toTcpDumpArgs());
	    script.append("\'");
	}
	else  script.append(testFileName);
    }

    protected void appendRestArgs() {
	script.append(' ');
	script.append(server);
	script.append(' ');
	script.append(serverPort);
	//	script.append('\"');
    }

//     protected void createLocalFileScript() {

//     }

    protected void createRemoteLiveScript() {
	 // append tcpdump stuff:
	script=new StringBuffer("rsh -l mguenter ");
	script.append(t_location);
	script.append(" /home/mguenter/CPP/tComponent ");

    } // rsh -l mguenter fermat $remotePth/tComponent $1 $2 $3

    protected void startDummyTcomponent() {
	System.out.println("Starting dummy T-Component.");
	(new Thread(t_dummy)).run();
    }
    public static void main(String[] args) {
	// testing
// 	int[] dsts=new int[2];
// 	dsts[0]=AddressCalculator.parseIPaddress("130.92.64.15");
// 	dsts[1]=AddressCalculator.parseIPaddress("130.92.65.25");
// 	Range proto=new Range(0,10);
// 	Range from=new Range(AddressCalculator.parseIPaddress("130.92.64.11"),(short) 24);
// 	List to=new List(dsts);
// 	Filter f=new Filter(from,to,proto,-1,100);
// 	T_Configurator t=new T_Configurator("balu", 3330, f, true, false);
// 	(new Thread(t)).start(); 

	    //System.out.println(f1.toTcpDumpArgs());
    }

}
