package node.services;

import node.*;
import java.util.*;
/* Written by Manuel Gnter. All rights reserved, no warranties */



public class ServiceDatabase {
    // Authoritative source to map service numbers to descriptions and handlers.
    // The method 'createDatabase()' stores template service handlers for all
    // known services. The method 'services' creates a customised service list
    // for an execution environment.

    protected static int defServ=5; // The first numbers (0..defServ-1)are assigned to default services. 
    protected static long maxUses=-1; // The first numbers are assigned to default services. 
    protected ServiceList database=new ServiceList(new Hashtable()); // The table with all
    // service handlers of supported services (handlers are used as factories for
    // the per-agent handlers).

    // The temporary context info:
    protected Node node;
    protected ExecutionEnvironment execEnv;

    public ServiceDatabase(Node n) {
	maxUses=n.MAX_USES_PER_SERVICE();
	createDatabase();
    }

    public ServiceList services(Vector numbers, Node n, ExecutionEnvironment ev) {
	// Generates a correct service list with handlers from a list of
	// service numbers.
	// Only the numbers of the non-default services must be given.
	node=n;  // Set the context to be used in createEntry.
	execEnv=ev;
	ServiceList ret=new  ServiceList(new Hashtable());
	setDefaultServiceHandlers(ret);
	Integer i;
	for(Enumeration e=numbers.elements(); e.hasMoreElements(); ) {
	    i=(Integer) e.nextElement();
	    ret.addService(createEntry(i.intValue()));
	}
	ret.initListPointer(); // Allows access between services of this list.
	return ret;	
    }

    protected void setDefaultServiceHandlers(ServiceList sl) {
	for(int i=0; i<defServ; i++) {
	    sl.addService(createEntry(i));
	}
    }

    protected void createDatabase() {
	// Here is the mapping, add new service handler classes here.
	// The type (int alias, gotten from ServiceConsts goes into the
	// not-complete constructor of the handler.
	// This instance is then used as a factory/template.
	// Note, that the default services up to number 'defServ' must
	// be added here.

	// Note: the constructors of the service handlers implement the
	// service type (important).
	// The service description communicate the format of the Service
	// object expected. See ServiceDescription


	database.addService(1, new NodeStopHandler(ServiceConsts.STOP_NODE()), new ServiceDescription("Killing the node (privileged operation).","null","none"));

	database.addService(1, new AllAgentsStopHandler(ServiceConsts.STOP_AGENTS()), new ServiceDescription("Killing all running agents (privileged operation).","null","none"));

 	database.addService(1, new ChangeChannel(ServiceConsts.CHANGE_CONNECTION()), new ServiceDescription("Opens a new connection over which the HomeChannel service sends.","Vector(4-5): Integer: String receiver-address (dns), Integer port, Boolean encrypt, Boolean authenticate,  String receiverID (if encrypt).","Vector(1): Boolean ok"));

	database.addService(5, new WakeUpHandler(ServiceConsts.WAKE_UP()), new ServiceDescription("Delivers a wake up call every x giffies (100ms) for n times.","Vector(2): Integer x, Integer n","Integer callNumber"));
	// Note: each handler opens an own thread-> restrict the service to limited nrs

	database.addService(1, new WhereAmIhandler(ServiceConsts.WHERE_AM_I()), new ServiceDescription("Gives ID infos about the executing node.","null","Vector(3): String nodeID, String organisation, String DNS-address"));

	database.addService(maxUses, new HomeChannel(ServiceConsts.SEND()), new ServiceDescription("Sending on connection of agent arrival.", "Vector(1): clientserver.Result", "none"));

	database.addService(1,  new AgentStopHandler(ServiceConsts.STOP_ME()) , new ServiceDescription("The agent self stopping service.", "null.", "none"));

    }

    protected ServiceListEntry createEntry(int serviceNr) {
	// The templates/factories in the database together with the current
	// execution context are used to create an entry of the individual
	// service list.
	ServiceListEntry factory=database.getServiceListEntry(serviceNr);
	if (factory==null) {
	    System.err.println("Warning: service authorised but not installable: "+serviceNr);
	    return null;
	}
	return new ServiceListEntry(factory.uses() ,
				    factory.handler().newServiceHandlerFor(execEnv, node) ,
				    factory.description() ); // In copy constructor einbuendeln? Zusaetzlich factory mitgeben, so dass services sich von vorherigen initiieren koennen.
    }

    public String toString() {
	StringBuffer buf= new StringBuffer();
	buf.append("Service Database:\n");
	buf.append("Default services from 0 to ");
	buf.append(defServ);
	buf.append(". Max. serving #: "+maxUses);
	buf.append(". Services:\n");
	buf.append(database.toString());
	return new String(buf);

    }
}
