package node.services;

import node.*;
import clientserver.*;
import java.util.Vector;


public class HomeChannel extends ServiceHandler {
    // Encapsulates the ProtocolObjectSenderReceiver which is connected to the home-app;
  
    
    protected ServerPOSenderReceiver posr;
    protected boolean ok=true;
    protected boolean last=false; // When the agent sends a Result flagged as
    // 'last' we complain about further sendings.

    ServiceHandler newServiceHandlerFor(ExecutionEnvironment ev, Node n) {
	return new HomeChannel(n, ev, type);
    }

    HomeChannel(int type) {
	// Non-complete constructor , which is then used as a factory.
	this.type=type;
    };
 
    HomeChannel(Node n, ExecutionEnvironment ev, int typ) {
	// Only the ServiceDatabase uses the constructor, to assure the type consistency.
	type=typ;
	posr=ev.posr();
    }


   synchronized boolean startService(Service req) {
	if(!ok) return false; // performance improvement
	if(last) {
	    System.err.println("Agent tries to send after last message was sent.");
	    return false;
	}
	Vector v=req.arguments();
	if(v==null) {
	    System.err.println("Homesending: empty arguments in the request.");
	    return false;
	}
	Object arg=v.firstElement();
	if(!(arg instanceof Result)) {
	    System.err.println("Homesending of something else than a Result.");
	    return false;
	}
	Result r=(Result) arg;
	if (r.isLast()) last=true;
	ok=posr.send(r);
	return ok;
    }

    void stopService() {
	if(!last && ok) {// if the client was not already told that the connection
	    // is not used further, and the connection is still ok, we do it here.
	    System.out.println("last="+last+" ok="+ok);
	    posr.send(new Result("HomeChannel: closing the connection...", true));
	    last=true;
	}
    }

    synchronized void replacePOSR(ServerPOSenderReceiver np) {
	// The ChangeChannel service handler changes the posr.

	//stopService(); // softer version, but lead to errors when the client
	// already closed the channel (e.g. when forwarding).

	(new Thread(new POSRcloser(posr))).start(); // Closes the old connection
	// after a short delay. The client protocol should be aware of that.
	posr=np;
	last=false;// Enable the service again.
    }

}
