package topology;

import java.util.*;

public class NodeLookupTable extends TopologyTable {
    // Contains all infos about location and name of all nodes.

    public NodeLookupTable(String file) {
	TableParser p=new TableParser(new NodeRecordParser());
	TopologyTable tab=p.readTxtFromDisk(file);
	initFrom(tab);
    }

    public NodeRecord recordOf(String id) {
	Object o=table.get(id);
	if(o==null) return null;
	return (NodeRecord) o; 
    }

    public Vector recordsOf(Vector ids) {
	// Convenient access, since eg the neighbors are a list of node ids.
	Vector ret=new Vector();
	NodeRecord norec;
	String id;
	for(Enumeration e=ids.elements(); e.hasMoreElements();) {
	    id=(String) e.nextElement();
	    norec=recordOf(id);
	    if(norec!=null)
		ret.addElement(norec);
	}
	return ret;
    }

    public Vector recordsOf(NeighborRecord neigr) {
	// Sub-optimal: could be optimized a lot.
	return recordsOf(neigr.neighbors);
    }


    // Helpers to get users- resp. node only neighbors. (Selection only)

    public NeighborRecord neighborNodes(NeighborRecord allNeighbors) {
	return neighbors(allNeighbors,false);
    }
    public NeighborRecord directUsers(NeighborRecord allNeighbors) {
	return neighbors(allNeighbors,true);
    }

    protected NeighborRecord neighbors(NeighborRecord allNeigh, boolean users) {
	// Creates a separate list with either all users or all nodes.
	NeighborRecord nr=allNeigh;
	if (nr==null) return null;
	Vector selected=new Vector();
	String nid;
	NodeRecord norec;
	for(Enumeration e=nr.neighbors.elements(); e.hasMoreElements();) {
	    nid=(String) e.nextElement();
	    norec=recordOf(nid);
	    if(norec != null)
		if(norec.isUser==users)
		    selected.addElement(nid);
	}
	nr=new NeighborRecord(allNeigh.key, selected);
	return nr;
    }


    public  String toString() {
	return "NodeLookupTable: "+super.toString();
    }


}
