package topology;

import utils.*;

import java.io.*;
import java.util.*;
 
public class NodeRecordParser extends TableRecordParser {
    // Parses and outputs NodeRecord
    // Currend record file structure is: 'nodeID: dnsName : ip : port : routingPort : organisation'

    protected PrimitiveTypeParser ptParser=new PrimitiveTypeParser();

    protected void write(TableRecord rec) throws IOException {
	if(rec instanceof NodeRecord) {
	    String s=" "+separator+" ";
	    NodeRecord nr=(NodeRecord) rec;
	    output.write(nr.dnsName);
	    output.write(s);
	    if(nr.ip != 0)
		output.write(AddressCalculator.toString(nr.ip));	
	    output.write(s);
	    output.write((new Integer(nr.port)).toString());
	    output.write(s);
	    output.write((new Integer(nr.routingPort)).toString());
	    output.write(s);
	    output.write(nr.organisation);
	    output.write(s);
	    output.write((new Boolean(nr.isUser)).toString());
	} else {
	    System.err.println("Wrong class for record entry: "+rec);
	}
    }
   
    protected TableRecord parseRecord() {
	NodeRecord nr=new NodeRecord();
	String s=giveNextToken(parser);
	boolean hasDns=true;
	boolean hasIP=true;

	if(s.equals("")) {
	    hasDns=false;
	} else {
	    nr.dnsName=ptParser.toDNS(s);
	}
	s=giveNextToken(parser);
	if(s.equals("")) 
	    hasIP=false;
	else
	    nr.ip=AddressCalculator.parseIPaddress(s);
	String ipOfdns="";
	if(hasDns) // Lookup the ip
	    ipOfdns=AddressCalculator.dnsToIP(nr.dnsName);
	if(hasDns && hasIP)
	    if(!AddressCalculator.toString(nr.ip).equals(ipOfdns)) // consistency check
		System.err.println("NodeRecordParser warning: IP does not fit for: "+nr.dnsName);
	if(hasDns && !hasIP) 
	    nr.ip=AddressCalculator.parseIPaddress(ipOfdns);
	//if(!hasDns && hasIP) //unfortunatelly there's no routine for this case.
	if(!hasDns && !hasIP)
	    System.err.println("NodeRecordParser error: No DNS and no IP provided!");	 
         
	s=giveNextToken(parser);
	nr.port=ptParser.toPort(s);
	s=giveNextToken(parser);
	nr.routingPort=ptParser.toPort(s);
	s=giveNextToken(parser);
	nr.organisation=s;
	s=giveNextToken(parser);
	nr.isUser=ptParser.toBoolean(s);
	return nr;
    }

    protected String giveNextToken(StringTokenizer parser) {
	String ret=super.giveNextToken(parser);
	if(ret==null) System.err.println("NodeRecordParser error: didn't get mandatory item!");
	return ret;
    }

}
