package topology;

import java.io.*;
import java.util.*;


public abstract class TableRecordParser {
    // Reads and writes lines of tables from/to a reader/writer (stream).

    protected static final String separator=":";
    protected StringTokenizer parser; // For splitting between the separators.
    protected BufferedReader input=null;
    protected BufferedWriter output=null;

    public void setReader(Reader r) {
	// Should be called before 'read'.
	input=new BufferedReader(r);
    }

    public void setWriter(Writer w) {
	// Should be called before 'write'.
	output=new BufferedWriter(w);
    }

    public void write(String id, TableRecord rec) {
	try {
	    if(output==null) {
		System.err.println("TableRecordParser error: no writer set.");
		return;
	    }
	    String s="\t"+separator+"\t";
	    output.write(id);
	    output.write(s);
	    write(rec);
	    output.write('\n');
	    output.flush();
	} catch (IOException e) {
	    System.err.println("TableRecordParser write error: "+e);
	}
    }

    protected abstract void write(TableRecord rec) throws IOException;
   

    public TableRecord read()  {
	try {
	    if(input==null) {
		System.err.println("TableRecordParser read error: no reader set.");
		return null;
	    }
	    String line=input.readLine();
	    if (line==null) return null; // End of data.
	    return parseLine(line);
	} catch (Exception e) {
	    System.err.println("TableRecordParser error: "+e);
	    e.printStackTrace();
	}
	return null;
    }

    protected TableRecord parseLine(String line) {
	// Parses one line of input into a profile object.
	TableRecord ret=null;
	parser=new StringTokenizer(line,separator+"\n");
	String id=null;

	id=giveNextToken(parser);
	ret= parseRecord();
	ret.key=id;

	return ret;
    }

    protected abstract TableRecord parseRecord() ;

    protected String giveNextToken(StringTokenizer parser) {
	String ret=null;
	try {
	    ret=parser.nextToken();
	    ret=ret.trim(); // eliminate leading/trailing whitespaces.
	} catch(Exception e) {}
	return ret;
    }

}
