package capsule;

import node.*;
import node.services.*;
import utils.Hash;

import java.io.*;
import java.util.*;

public abstract class HashAgent extends MyAgent {
    protected int from,to; // Byte position where hash is calculated. to<from means up to the ende of the packet.
    protected byte[] match; // Bytes that the hash must match.
    protected int matchLen; // This many bits are compared.
    protected long matched=0; // can be used by subclasses as counter.
    protected byte[] hash;
    protected Hash hashFct;

    protected  int fullbytes;  // matchLen is transformed into matching bytes and some broken bytes (bits).
    protected  int brokenbits;// These ivs are set in the init method.

    protected boolean match(IPPacket p) {
	byte[] content=p.bytes;
	int len=content.length;
	int byts=to-from;
	if ((byts<0) || (to>len)) byts=len-from;
	return match(content,from, byts);
    }

    protected boolean udpPayloadMatch(IPPacket p) {
	// Hashes the whole (available) udp payload.
	byte[] content=p.bytes;
	setUDPto(p);
	int len=content.length;
	int byts;
	if (to>len) byts=len-from;
	else byts=to-from;
	return match(content,from, byts);
    }

    protected boolean match(byte[] content, int from, int byts) {
	System.out.println("Mtch from: "+from+" bytes: "+byts);
	hash=hashFct.hash(content, from , byts);
	return matchToHash();
    }

    protected void setUDPto(IPPacket p) {
	// for hashing on the udp payload.
	// overrides the user setting of 'to'.
	int paylen=p.toPositiveNumber(24, 2);
	to=20+paylen;
    }

    protected boolean matchToHash() {
	// The match is calc'd byte wise, broken bytes are matched from most sig bit downwards.
	System.out.println("Agent's hash: "+printHash(match));
	System.out.println("Packet's hash: "+printHash(hash));

	int i=0;
	for(; i<fullbytes; i++) {
	    if(hash[i]!=match[i]) return false;
	}
	if(brokenbits!=0) {
	    int shift=8-brokenbits;
	    if((hash[i]>>shift)==(match[i]>>shift)) {
		matched++;
		return true;
	    } else
		return false;
	}
	matched++;
	return true;
    }

    public boolean init(Vector initVector) {
	initHash();
	if(!checkHashInit()) return false;
	fullbytes=matchLen>>3;
	brokenbits=matchLen&7;
	//System.out.println("FB: "+fullbytes+"BB: "+brokenbits);
	return super.init(initVector);
    }

    protected String printHash(byte[] hsh) {
	StringBuffer buf=new StringBuffer();
	for(int i=0; i<hsh.length; i++) {
	    buf.append(hsh[i]);
	    buf.append(' ');
	}
	return new String(buf);
    }

    protected boolean checkHashInit() {
	if(from<0) {
	    System.err.println("HashAgent init error: hash range is too small!");
	    return false;
	}
	if(hashFct==null){
	    System.err.println("HashAgent init error: Hash function not initiallized!");
	    return false;
	}
	if((matchLen<0) || (matchLen>128)) {
	    System.err.println("HashAgent init error: matchLen should be between 0 and 128!");
	    return false;
	}
	if(matchLen>8*match.length){
	    System.err.println("HashAgent init error: matchLen should not be longer than match byte array!");
	    return false;
	}
	return true;
    }

    public abstract void  initHash();
	// Init the IVs here.
  
}
