package utils;

import java.util.*;

public class ObjectQueue {
    // A synchronized queue for objects. Is used as store for IPPacket and Result objects
    // (In- resp. outqueue).
    protected int qlen;
    protected Object[] queue;
    protected int in=0;
    protected int out=0;
    protected int used=0;
    
    protected int state=0;


    public ObjectQueue(int len) {
	qlen=len;
	queue = new Object[qlen];
    }

    public int capacity() {return qlen;}
    public synchronized int used() {return used;}

    public synchronized int free() {return qlen-used;}
    public synchronized boolean full() {return qlen==used;}
    public synchronized boolean empty() {return used==0;}

    public synchronized int changedSinceLast() {
	// For RessourceControler to see how the queue has changed.
	int old=state;
	state=used;
	return used-old;
    }

    synchronized public void put(Object ip) {
	while(used==qlen) // queue is full.
	    try{wait();} catch(InterruptedException ex) {};
	//System.out.print("-P-");
	queue[in]=ip;
	in=(in+1)%qlen;
	if(used++ ==0) notifyAll();
    }


    synchronized public Object get() {
	Object ip;
	while(used==0) // queue is empty.
	    try{wait();} catch(InterruptedException ex) {};
	//System.out.print("-G-");
	ip=queue[out];
	out=(out+1)%qlen;
	if(used-- ==qlen) notifyAll();
	return ip;
    }

    
}
